//****************************************************************************
//	
//	Copyright 2018 by WinSystems Inc.
//
//****************************************************************************
//
//	Name	 : getvolt_irq.cpp
//
//	Project	 : PCM-MIO Console Application
//
//	Author	 : Paul DeMetrotion
//
//****************************************************************************
//
//	  Date		  Rev	                Description
//	--------    -------	   ---------------------------------------------
//	03/09/18	  1.0		Original Release
//
//****************************************************************************

#include <windows.h>
#include <conio.h>
#include <stdio.h>
#include <stdlib.h>
#include <tchar.h>
#include <strsafe.h>
#include "pcmmioDLL.h"

DWORD WINAPI AdcConversion1(LPVOID lpParam);
DWORD WINAPI AdcConversion2(LPVOID lpParam);

#define MAJOR_VER   1
#define MINOR_VER   0
#define DEVICE      1

volatile int conv_count1 = 0;
volatile int conv_count2 = 0;
volatile int exit_flag = 0;

int _tmain(int argc, _TCHAR* argv[])
{
    HANDLE hThread[2];
    DWORD dwThreadId[2];
    unsigned long timeout = 0x1000; // 4096 msec
    int dllReturn = SUCCESS;

    printf("PCM-MIO Application : getvolt_irq\n");
    printf("Version %d.%d\n\n", MAJOR_VER, MINOR_VER);

    dllReturn = InitializeSession(DEVICE);

    if (dllReturn)
    {
        printf("Error initializing session.\n");
        exit(dllReturn);
    }
    else
    {
        printf("PCM-MIO device opened.\n");
    }

    // configure adc channels
    for (int i = 0; i < 16; i++) {
        dllReturn = AdcSetChannelMode(DEVICE, i, ADC_SINGLE_ENDED, ADC_BIPOLAR, ADC_TOP_10V);

        if (dllReturn)
        {
            printf("Error configuring adc channel %d.\n", i);
            exit(dllReturn);
        }
    }

    // enable interrupts for adc1 and adc2
    dllReturn = AdcEnableInterrupt(DEVICE, 0); // channel 0 will enable adc1 interrupts

    if (dllReturn)
    {
        printf("Error enabling interrupts for ADC1.\n");
        exit(dllReturn);
    }

    dllReturn = AdcEnableInterrupt(DEVICE, 8); // channel 8 will enable adc2 interrupts

    if (dllReturn)
    {
        printf("Error enabling interrupts for ADC1.\n");
        exit(dllReturn);
    }

    // Create threads to begin execution
    hThread[0] = CreateThread(
        NULL,                   // default security attributes
        0,                      // use default stack size  
        AdcConversion1,         // thread function name
        &timeout,               // argument to thread function
        0,                      // use default creation flags 
        &dwThreadId[0]);        // returns the thread identifier 

    // if CreateThread fails, terminate execution 
    if (hThread[0] == NULL)
    {
        printf("Cannot create thread!\n");
        exit(1);
    }

    hThread[1] = CreateThread(
        NULL,                   // default security attributes
        0,                      // use default stack size  
        AdcConversion2,         // thread function name
        &timeout,               // argument to thread function
        0,                      // use default creation flags 
        &dwThreadId[1]);        // returns the thread identifier 

    // if CreateThread fails, terminate execution 
    if (hThread[1] == NULL)
    {
        printf("Cannot create thread!\n");
        exit(1);
    }

    // print heartbeat
    do {
        printf(" * ");

        Sleep(10);
    } while (!_kbhit()) ; // wait for key stroke

    int ch = _getch();

    // done so set flag
    exit_flag = 1;
            
    // disable interrupts for adc1 and adc2
    dllReturn = AdcDisableInterrupt(DEVICE, 0); // channel 0 will disable adc1 interrupts

    if (dllReturn)
    {
        printf("Error disabling interrupts for ADC1.\n");
        exit(dllReturn);
    }

    dllReturn = AdcDisableInterrupt(DEVICE, 8); // channel 8 will disable adc2 interrupts

    if (dllReturn)
    {
        printf("Error disabling interrupts for ADC2.\n");
        exit(dllReturn);
    }

    // Wait for threads to terminate
    WaitForSingleObject(hThread[0], INFINITE);
    WaitForSingleObject(hThread[1], INFINITE);

    // close thread handles
    CloseHandle(hThread[0]);
    CloseHandle(hThread[1]);

    // finish
    dllReturn = CloseSession(DEVICE);

    if (dllReturn)
    {
        printf("Unable to close device.\n");
        exit(dllReturn);
    }
    else
        printf("\nPCM-MIO device closed.\n");

    // display our event count total
    printf("\nConversion count ADC1 = %d\r", conv_count1);
    printf("\nConversion count ADC2 = %d\r", conv_count2);
}

DWORD WINAPI AdcConversion1(LPVOID lpParam)
{
    int channel = 0;
    unsigned short chValue;
    float chVoltage;
    unsigned long *timeout = (unsigned long*) lpParam;
    int dllReturn = SUCCESS;

    // first conversion at channel 0 to align data properly
    dllReturn = AdcWaitForConversion(DEVICE, 0, &chValue, *timeout);

    // use interrupts now
    while (!exit_flag)
    {
        // put THIS process to sleep until conversion completes
        dllReturn = AdcWaitForConversion(DEVICE, (channel + 1) % 8, &chValue, *timeout);

        // restart loop if error
        if (dllReturn) {
            printf("Error %d ...\n", dllReturn);
        }
        else {
            AdcConvertToVolts(DEVICE, channel, chValue, &chVoltage);

            printf("\nADC Channel %d voltage read is %.5f.\n", channel, chVoltage);

            conv_count1++;
        }

        if (++channel == 8) channel = 0;

        Sleep(50);
    }

    return 0;
}

DWORD WINAPI AdcConversion2(LPVOID lpParam)
{
    int channel = 8;
    unsigned short chValue;
    float chVoltage;
    unsigned long *timeout = (unsigned long*)lpParam;
    int dllReturn = SUCCESS;

    // first conversion at channel 8 to align data properly
    dllReturn = AdcWaitForConversion(DEVICE, channel, &chValue, *timeout);

    // use interrupts now
    while (!exit_flag)
    {
        // put THIS process to sleep until conversion completes
        dllReturn = AdcWaitForConversion(DEVICE, ((channel + 1) % 8) + 8, &chValue, *timeout);

        // restart loop if error
        if (dllReturn) {
            printf("Error %d ...\n", dllReturn);
        }
        else {
            AdcConvertToVolts(DEVICE, channel, chValue, &chVoltage);

            printf("\nADC Channel %d voltage read is %.5f.\n", channel, chVoltage);

            conv_count2++;
        }

        if (++channel == 16) channel = 8;

        Sleep(50);
    }

    return 0;
}
